"""
Python mapping for the CryptoTokenKit framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""

import sys

import Foundation
import objc
from CryptoTokenKit import _metadata
from CryptoTokenKit import _CryptoTokenKit

sys.modules["CryptoTokenKit"] = mod = objc.ObjCLazyModule(
    "CryptoTokenKit",
    "com.apple.CryptoTokenKit",
    objc.pathForFramework("/System/Library/Frameworks/CryptoTokenKit.framework"),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "objc": objc,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
    },
    (_CryptoTokenKit, Foundation),
)


del sys.modules["CryptoTokenKit._metadata"]



import objc

from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure
import CryptoTokenKit


class TestTKSmartCard(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(CryptoTokenKit.TKSmartCardPINCharset)
        self.assertIsEnumType(CryptoTokenKit.TKSmartCardPINCompletion)
        self.assertIsEnumType(CryptoTokenKit.TKSmartCardPINConfirmation)
        self.assertIsEnumType(CryptoTokenKit.TKSmartCardPINEncoding)
        self.assertIsEnumType(CryptoTokenKit.TKSmartCardPINJustification)
        self.assertIsEnumType(CryptoTokenKit.TKSmartCardSlotState)

    @min_os_level("10.10")
    def testClasses(self):
        self.assertIsInstance(CryptoTokenKit.TKSmartCardSlotManager, objc.objc_class)
        self.assertIsInstance(CryptoTokenKit.TKSmartCardSlot, objc.objc_class)
        self.assertIsInstance(CryptoTokenKit.TKSmartCard, objc.objc_class)

    @min_os_level("10.10")
    def testMethods(self):
        self.assertArgIsBlock(
            CryptoTokenKit.TKSmartCardSlotManager.getSlotWithName_reply_, 1, b"v@"
        )

        self.assertResultIsBOOL(CryptoTokenKit.TKSmartCard.valid)
        self.assertResultIsBOOL(CryptoTokenKit.TKSmartCard.sensitive)
        self.assertArgIsBOOL(CryptoTokenKit.TKSmartCard.setSensitive_, 0)
        self.assertArgIsBlock(
            CryptoTokenKit.TKSmartCard.beginSessionWithReply_,
            0,
            b"v" + objc._C_NSBOOL + b"@",
        )
        self.assertArgIsBlock(
            CryptoTokenKit.TKSmartCard.transmitRequest_reply_, 1, b"v@@"
        )
        self.assertResultIsBOOL(CryptoTokenKit.TKSmartCard.useExtendedLength)
        self.assertArgIsBOOL(CryptoTokenKit.TKSmartCard.setUseExtendedLength_, 0)
        self.assertArgIsBlock(
            CryptoTokenKit.TKSmartCard.sendIns_p1_p2_data_le_reply_, 5, b"v@S@"
        )

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertArgIsBlock(
            CryptoTokenKit.TKSmartCardUserInteraction.runWithReply_, 0, b"vZ@"
        )
        self.assertResultIsBOOL(CryptoTokenKit.TKSmartCardUserInteraction.cancel)

        self.assertResultIsBOOL(
            CryptoTokenKit.TKSmartCardUserInteractionForConfirmation.result
        )
        self.assertArgIsBOOL(
            CryptoTokenKit.TKSmartCardUserInteractionForConfirmation.setResult_, 0
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(
            CryptoTokenKit.TKSmartCard.inSessionWithError_executeBlock_
        )
        self.assertArgIsOut(
            CryptoTokenKit.TKSmartCard.inSessionWithError_executeBlock_, 0
        )
        self.assertArgIsBlock(
            CryptoTokenKit.TKSmartCard.inSessionWithError_executeBlock_, 1, b"Zo^@"
        )

        self.assertArgIsOut(
            CryptoTokenKit.TKSmartCard.sendIns_p1_p2_data_le_sw_error_, 5
        )
        self.assertArgIsOut(
            CryptoTokenKit.TKSmartCard.sendIns_p1_p2_data_le_sw_error_, 6
        )

    @min_os_level("10.11")
    @expectedFailure
    def testMethods10_11_missing(self):
        self.assertResultIsBOOL(
            CryptoTokenKit.TKSmartCardSlotScreen.displayMessage_x_y_duration_clearScreen_
        )
        self.assertArgIsBOOL(
            CryptoTokenKit.TKSmartCardSlotScreen.displayMessage_x_y_duration_clearScreen_,
            4,
        )

    @min_os_level("10.10")
    def testConstants(self):
        self.assertEqual(CryptoTokenKit.TKSmartCardSlotStateMissing, 0)
        self.assertEqual(CryptoTokenKit.TKSmartCardSlotStateEmpty, 1)
        self.assertEqual(CryptoTokenKit.TKSmartCardSlotStateProbing, 2)
        self.assertEqual(CryptoTokenKit.TKSmartCardSlotStateMuteCard, 3)
        self.assertEqual(CryptoTokenKit.TKSmartCardSlotStateValidCard, 4)

        self.assertEqual(CryptoTokenKit.TKSmartCardPINCharsetNumeric, 0)
        self.assertEqual(CryptoTokenKit.TKSmartCardPINCharsetAlphanumeric, 1)
        self.assertEqual(CryptoTokenKit.TKSmartCardPINCharsetUpperAlphanumeric, 2)

        self.assertEqual(CryptoTokenKit.TKSmartCardPINEncodingBinary, 0)
        self.assertEqual(CryptoTokenKit.TKSmartCardPINEncodingASCII, 1)
        self.assertEqual(CryptoTokenKit.TKSmartCardPINEncodingBCD, 2)

        self.assertEqual(CryptoTokenKit.TKSmartCardPINJustificationLeft, 0)
        self.assertEqual(CryptoTokenKit.TKSmartCardPINJustificationRight, 1)

        self.assertEqual(CryptoTokenKit.TKSmartCardPINCompletionMaxLength, 1 << 0)
        self.assertEqual(CryptoTokenKit.TKSmartCardPINCompletionKey, 1 << 1)
        self.assertEqual(CryptoTokenKit.TKSmartCardPINCompletionTimeout, 1 << 2)

        self.assertEqual(CryptoTokenKit.TKSmartCardPINConfirmationNone, 0)
        self.assertEqual(CryptoTokenKit.TKSmartCardPINConfirmationNew, 1 << 0)
        self.assertEqual(CryptoTokenKit.TKSmartCardPINConfirmationCurrent, 1 << 1)

        # Deprecated:
        self.assertEqual(CryptoTokenKit.TKSmartCardNoSlot, 0)
        self.assertEqual(CryptoTokenKit.TKSmartCardSlotEmpty, 1)
        self.assertEqual(CryptoTokenKit.TKSmartCardSlotProbing, 2)
        self.assertEqual(CryptoTokenKit.TKSmartCardSlotMuteCard, 3)
        self.assertEqual(CryptoTokenKit.TKSmartCardSlotValidCard, 4)

    @min_os_level("10.11")
    def testProtocols(self):
        self.assertProtocolExists("TKSmartCardUserInteractionDelegate")



from PyObjCTools.TestSupport import TestCase, min_os_level
import CryptoTokenKit


class TestTKTokenConfiguration(TestCase):
    @min_os_level("10.15")
    def test_methods(self):
        self.assertArgIsOut(
            CryptoTokenKit.TKTokenConfiguration.keyForObjectID_error_, 1
        )
        self.assertArgIsOut(
            CryptoTokenKit.TKTokenConfiguration.certificateForObjectID_error_, 1
        )



from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import CryptoTokenKit


class TestTKTokenHelper(CryptoTokenKit.NSObject):
    def tokenSession_beginAuthForOperation_constraint_error_(self, s, o, c, e):
        return 1

    def tokenSession_supportsOperation_usingKey_algorithm_(self, s, o, k, a):
        return 1

    def tokenSession_signData_usingKey_algorithm_error_(self, s, d, k, a, e):
        return 1

    def tokenSession_decryptData_usingKey_algorithm_error_(self, s, d, k, a, e):
        return 1

    def tokenSession_performKeyExchangeWithPublicKey_usingKey_algorithm_parameters_error_(
        self, s, pk, k, a, p, e
    ):
        return 1

    def token_createSessionWithError_(self, t, e):
        return 1

    def tokenDriver_tokenForConfiguration_error_(self, a, b, c):
        pass


class TestTKToken(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(CryptoTokenKit.TKTokenOperation)

    @min_os_level("10.12")
    def testConstants(self):
        self.assertEqual(CryptoTokenKit.TKTokenOperationNone, 0)
        self.assertEqual(CryptoTokenKit.TKTokenOperationReadData, 1)
        self.assertEqual(CryptoTokenKit.TKTokenOperationSignData, 2)
        self.assertEqual(CryptoTokenKit.TKTokenOperationDecryptData, 3)
        self.assertEqual(CryptoTokenKit.TKTokenOperationPerformKeyExchange, 4)

    @min_os_level("10.12")
    def testMethods(self):
        self.assertResultIsBOOL(CryptoTokenKit.TKTokenKeyAlgorithm.isAlgorithm_)
        self.assertResultIsBOOL(CryptoTokenKit.TKTokenKeyAlgorithm.supportsAlgorithm_)

        self.assertResultIsBOOL(CryptoTokenKit.TKTokenAuthOperation.finishWithError_)
        self.assertArgIsOut(CryptoTokenKit.TKTokenAuthOperation.finishWithError_, 0)

    @min_sdk_level("10.12")
    def testProtocols(self):
        self.assertProtocolExists("TKTokenSessionDelegate")
        self.assertProtocolExists("TKTokenDelegate")
        self.assertProtocolExists("TKTokenDriverDelegate")

    @min_sdk_level("10.12")
    def testProtocolMethods(self):
        self.assertArgHasType(
            TestTKTokenHelper.tokenSession_beginAuthForOperation_constraint_error_,
            3,
            b"o^@",
        )
        self.assertResultIsBOOL(
            TestTKTokenHelper.tokenSession_supportsOperation_usingKey_algorithm_
        )
        self.assertArgHasType(
            TestTKTokenHelper.tokenSession_signData_usingKey_algorithm_error_, 4, b"o^@"
        )
        self.assertArgHasType(
            TestTKTokenHelper.tokenSession_decryptData_usingKey_algorithm_error_,
            4,
            b"o^@",
        )
        self.assertArgHasType(
            TestTKTokenHelper.tokenSession_performKeyExchangeWithPublicKey_usingKey_algorithm_parameters_error_,
            5,
            b"o^@",
        )

        self.assertArgHasType(
            TestTKTokenHelper.token_createSessionWithError_, 1, b"o^@"
        )
        self.assertArgHasType(
            TestTKTokenHelper.tokenDriver_tokenForConfiguration_error_, 2, b"o^@"
        )



from PyObjCTools.TestSupport import TestCase, min_sdk_level
import CryptoTokenKit


class TestTKSmartCardTokenHelper(CryptoTokenKit.NSObject):
    def tokenDriver_createTokenForSmartCard_AID_error_(self, d, sc, a, e):
        return 1


class TestTKSmartCardToken(TestCase):
    @min_sdk_level("10.12")
    def testProtocols(self):
        self.assertProtocolExists("TKSmartCardTokenDriverDelegate")

    @min_sdk_level("10.12")
    def testMethods(self):
        self.assertArgHasType(
            TestTKSmartCardTokenHelper.tokenDriver_createTokenForSmartCard_AID_error_,
            3,
            b"o^@",
        )



from PyObjCTools.TestSupport import TestCase
import CryptoTokenKit


class TestTKError(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(CryptoTokenKit.TKErrorCode)

    def testConstants(self):
        self.assertIsInstance(CryptoTokenKit.TKErrorDomain, str)

        self.assertEqual(CryptoTokenKit.TKErrorCodeNotImplemented, -1)
        self.assertEqual(CryptoTokenKit.TKErrorCodeCommunicationError, -2)
        self.assertEqual(CryptoTokenKit.TKErrorCodeCorruptedData, -3)
        self.assertEqual(CryptoTokenKit.TKErrorCodeCanceledByUser, -4)
        self.assertEqual(CryptoTokenKit.TKErrorAuthenticationFailed, -5)
        self.assertEqual(CryptoTokenKit.TKErrorCodeAuthenticationFailed, -5)
        self.assertEqual(CryptoTokenKit.TKErrorObjectNotFound, -6)
        self.assertEqual(CryptoTokenKit.TKErrorCodeObjectNotFound, -6)
        self.assertEqual(CryptoTokenKit.TKErrorTokenNotFound, -7)
        self.assertEqual(CryptoTokenKit.TKErrorCodeTokenNotFound, -7)
        self.assertEqual(CryptoTokenKit.TKErrorCodeBadParameter, -8)
        self.assertEqual(CryptoTokenKit.TKErrorCodeAuthenticationNeeded, -9)



from PyObjCTools.TestSupport import TestCase, min_os_level
import CryptoTokenKit


class TestTKTokenKeychainItem(TestCase):
    @min_os_level("10.12")
    def testMethods(self):
        self.assertResultIsBOOL(CryptoTokenKit.TKTokenKeychainKey.canDecrypt)
        self.assertResultIsBOOL(CryptoTokenKit.TKTokenKeychainKey.canSign)
        self.assertResultIsBOOL(CryptoTokenKit.TKTokenKeychainKey.canPerformKeyExchange)
        self.assertResultIsBOOL(CryptoTokenKit.TKTokenKeychainKey.isSuitableForLogin)

        self.assertArgIsBOOL(CryptoTokenKit.TKTokenKeychainKey.setCanDecrypt_, 0)
        self.assertArgIsBOOL(CryptoTokenKit.TKTokenKeychainKey.setCanSign_, 0)
        self.assertArgIsBOOL(
            CryptoTokenKit.TKTokenKeychainKey.setCanPerformKeyExchange_, 0
        )
        self.assertArgIsBOOL(CryptoTokenKit.TKTokenKeychainKey.setSuitableForLogin_, 0)

        self.assertArgIsOut(
            CryptoTokenKit.TKTokenKeychainContents.keyForObjectID_error_, 1
        )
        self.assertArgIsOut(
            CryptoTokenKit.TKTokenKeychainContents.certificateForObjectID_error_, 1
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import CryptoTokenKit


class TestTKTokenWatcher(TestCase):
    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertArgIsBlock(
            CryptoTokenKit.TKTokenWatcher.initWithInsertionHandler_, 0, b"v@"
        )
        self.assertArgIsBlock(
            CryptoTokenKit.TKTokenWatcher.addRemovalHandler_forTokenID_, 0, b"v@"
        )

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertArgIsBlock(
            CryptoTokenKit.TKTokenWatcher.setInsertionHandler_, 0, b"v@"
        )



# This file is generated by objective.metadata
#
# Last update: Sun Feb 20 18:47:18 2022
#
# flake8: noqa

import objc, sys
from typing import NewType

if sys.maxsize > 2**32:

    def sel32or64(a, b):
        return b

else:

    def sel32or64(a, b):
        return a


if objc.arch == "arm64":

    def selAorI(a, b):
        return a

else:

    def selAorI(a, b):
        return b


misc = {}
constants = """$TKErrorDomain$"""
enums = """$TKErrorAuthenticationFailed@-5$TKErrorCodeAuthenticationFailed@-5$TKErrorCodeAuthenticationNeeded@-9$TKErrorCodeBadParameter@-8$TKErrorCodeCanceledByUser@-4$TKErrorCodeCommunicationError@-2$TKErrorCodeCorruptedData@-3$TKErrorCodeNotImplemented@-1$TKErrorCodeObjectNotFound@-6$TKErrorCodeTokenNotFound@-7$TKErrorObjectNotFound@-6$TKErrorTokenNotFound@-7$TKSmartCardNoSlot@0$TKSmartCardPINCharsetAlphanumeric@1$TKSmartCardPINCharsetNumeric@0$TKSmartCardPINCharsetUpperAlphanumeric@2$TKSmartCardPINCompletionKey@2$TKSmartCardPINCompletionMaxLength@1$TKSmartCardPINCompletionTimeout@4$TKSmartCardPINConfirmationCurrent@2$TKSmartCardPINConfirmationNew@1$TKSmartCardPINConfirmationNone@0$TKSmartCardPINEncodingASCII@1$TKSmartCardPINEncodingBCD@2$TKSmartCardPINEncodingBinary@0$TKSmartCardPINJustificationLeft@0$TKSmartCardPINJustificationRight@1$TKSmartCardProtocolAny@65535$TKSmartCardProtocolNone@0$TKSmartCardProtocolT0@1$TKSmartCardProtocolT1@2$TKSmartCardProtocolT15@32768$TKSmartCardSlotEmpty@1$TKSmartCardSlotMuteCard@3$TKSmartCardSlotProbing@2$TKSmartCardSlotStateEmpty@1$TKSmartCardSlotStateMissing@0$TKSmartCardSlotStateMuteCard@3$TKSmartCardSlotStateProbing@2$TKSmartCardSlotStateValidCard@4$TKSmartCardSlotValidCard@4$TKTokenOperationDecryptData@3$TKTokenOperationNone@0$TKTokenOperationPerformKeyExchange@4$TKTokenOperationReadData@1$TKTokenOperationSignData@2$"""
misc.update(
    {
        "TKSmartCardPINCharset": NewType("TKSmartCardPINCharset", int),
        "TKErrorCode": NewType("TKErrorCode", int),
        "TKSmartCardPINConfirmation": NewType("TKSmartCardPINConfirmation", int),
        "TKSmartCardSlotState": NewType("TKSmartCardSlotState", int),
        "TKSmartCardPINCompletion": NewType("TKSmartCardPINCompletion", int),
        "TKTokenOperation": NewType("TKTokenOperation", int),
        "TKSmartCardPINEncoding": NewType("TKSmartCardPINEncoding", int),
        "TKSmartCardProtocol": NewType("TKSmartCardProtocol", int),
        "TKSmartCardPINJustification": NewType("TKSmartCardPINJustification", int),
    }
)
misc.update({})
aliases = {
    "TKSmartCardNoSlot": "TKSmartCardSlotStateMissing",
    "TKSmartCardSlotMuteCard": "TKSmartCardSlotStateMuteCard",
    "TKSmartCardSlotProbing": "TKSmartCardSlotStateProbing",
    "TKErrorTokenNotFound": "TKErrorCodeTokenNotFound",
    "TKSmartCardSlotValidCard": "TKSmartCardSlotStateValidCard",
    "TKErrorObjectNotFound": "TKErrorCodeObjectNotFound",
    "TKSmartCardSlotEmpty": "TKSmartCardSlotStateEmpty",
    "TKErrorAuthenticationFailed": "TKErrorCodeAuthenticationFailed",
}
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(
        b"NSObject",
        b"characterEnteredInUserInteraction:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"correctionKeyPressedInUserInteraction:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"invalidCharacterEnteredInUserInteraction:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"newPINConfirmationRequestedInUserInteraction:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"newPINRequestedInUserInteraction:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"oldPINRequestedInUserInteraction:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"token:createSessionWithError:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"@"}, 3: {"type": "o^@", "type_modifier": b"o"}},
        },
    )
    r(
        b"NSObject",
        b"token:terminateSession:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"tokenDriver:createTokenForSmartCard:AID:error:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"@"},
                4: {"type": b"@"},
                5: {"type": "o^@", "type_modifier": b"o"},
            },
        },
    )
    r(
        b"NSObject",
        b"tokenDriver:terminateToken:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"tokenDriver:tokenForConfiguration:error:",
        {
            "required": False,
            "retval": {"type": b"@"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"@"},
                4: {"type": "o^@", "type_modifier": b"o"},
            },
        },
    )
    r(
        b"NSObject",
        b"tokenSession:beginAuthForOperation:constraint:error:",
        {
            "required": False,
            "retval": {"type": b"@"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"q"},
                4: {"type": b"@"},
                5: {"type": "o^@", "type_modifier": b"o"},
            },
        },
    )
    r(
        b"NSObject",
        b"tokenSession:decryptData:usingKey:algorithm:error:",
        {
            "required": False,
            "retval": {"type": b"@"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"@"},
                4: {"type": b"@"},
                5: {"type": b"@"},
                6: {"type": "o^@", "type_modifier": b"o"},
            },
        },
    )
    r(
        b"NSObject",
        b"tokenSession:performKeyExchangeWithPublicKey:usingKey:algorithm:parameters:error:",
        {
            "required": False,
            "retval": {"type": b"@"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"@"},
                4: {"type": b"@"},
                5: {"type": b"@"},
                6: {"type": b"@"},
                7: {"type": "o^@", "type_modifier": b"o"},
            },
        },
    )
    r(
        b"NSObject",
        b"tokenSession:signData:usingKey:algorithm:error:",
        {
            "required": False,
            "retval": {"type": b"@"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"@"},
                4: {"type": b"@"},
                5: {"type": b"@"},
                6: {"type": "o^@", "type_modifier": b"o"},
            },
        },
    )
    r(
        b"NSObject",
        b"tokenSession:supportsOperation:usingKey:algorithm:",
        {
            "required": False,
            "retval": {"type": "Z"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"q"},
                4: {"type": b"@"},
                5: {"type": b"@"},
            },
        },
    )
    r(
        b"NSObject",
        b"validationKeyPressedInUserInteraction:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"TKSmartCard",
        b"beginSessionWithReply:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"Z"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"TKSmartCard",
        b"inSessionWithError:executeBlock:",
        {
            "retval": {"type": "Z"},
            "arguments": {
                2: {"type_modifier": b"o"},
                3: {
                    "callable": {
                        "retval": {"type": b"Z"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"o^@"}},
                    }
                },
            },
        },
    )
    r(
        b"TKSmartCard",
        b"sendIns:p1:p2:data:le:reply:",
        {
            "arguments": {
                7: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"S"},
                            3: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"TKSmartCard",
        b"sendIns:p1:p2:data:le:sw:error:",
        {"arguments": {7: {"type_modifier": b"o"}, 8: {"type_modifier": b"o"}}},
    )
    r(b"TKSmartCard", b"sensitive", {"retval": {"type": b"Z"}})
    r(b"TKSmartCard", b"setSensitive:", {"arguments": {2: {"type": b"Z"}}})
    r(b"TKSmartCard", b"setUseCommandChaining:", {"arguments": {2: {"type": b"Z"}}})
    r(b"TKSmartCard", b"setUseExtendedLength:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"TKSmartCard",
        b"transmitRequest:reply:",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(b"TKSmartCard", b"useCommandChaining", {"retval": {"type": b"Z"}})
    r(b"TKSmartCard", b"useExtendedLength", {"retval": {"type": b"Z"}})
    r(b"TKSmartCard", b"valid", {"retval": {"type": b"Z"}})
    r(
        b"TKSmartCardATR",
        b"initWithSource:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"i"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    }
                }
            }
        },
    )
    r(
        b"TKSmartCardSlotManager",
        b"getSlotWithName:reply:",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    }
                }
            }
        },
    )
    r(b"TKSmartCardUserInteraction", b"cancel", {"retval": {"type": "Z"}})
    r(
        b"TKSmartCardUserInteraction",
        b"runWithReply:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"Z"},
                            2: {"type": b"@"},
                        },
                    }
                }
            }
        },
    )
    r(
        b"TKSmartCardUserInteractionForConfirmation",
        b"result",
        {"retval": {"type": "Z"}},
    )
    r(
        b"TKSmartCardUserInteractionForConfirmation",
        b"setResult:",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(
        b"TKTokenAuthOperation",
        b"finishWithError:",
        {"retval": {"type": "Z"}, "arguments": {2: {"type_modifier": b"o"}}},
    )
    r(
        b"TKTokenConfiguration",
        b"certificateForObjectID:error:",
        {"arguments": {3: {"type_modifier": b"o"}}},
    )
    r(
        b"TKTokenConfiguration",
        b"keyForObjectID:error:",
        {"arguments": {3: {"type_modifier": b"o"}}},
    )
    r(b"TKTokenKeyAlgorithm", b"isAlgorithm:", {"retval": {"type": "Z"}})
    r(b"TKTokenKeyAlgorithm", b"supportsAlgorithm:", {"retval": {"type": "Z"}})
    r(
        b"TKTokenKeychainContents",
        b"certificateForObjectID:error:",
        {"arguments": {3: {"type_modifier": b"o"}}},
    )
    r(
        b"TKTokenKeychainContents",
        b"keyForObjectID:error:",
        {"arguments": {3: {"type_modifier": b"o"}}},
    )
    r(b"TKTokenKeychainKey", b"canDecrypt", {"retval": {"type": "Z"}})
    r(b"TKTokenKeychainKey", b"canPerformKeyExchange", {"retval": {"type": "Z"}})
    r(b"TKTokenKeychainKey", b"canSign", {"retval": {"type": "Z"}})
    r(b"TKTokenKeychainKey", b"isSuitableForLogin", {"retval": {"type": "Z"}})
    r(b"TKTokenKeychainKey", b"setCanDecrypt:", {"arguments": {2: {"type": "Z"}}})
    r(
        b"TKTokenKeychainKey",
        b"setCanPerformKeyExchange:",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(b"TKTokenKeychainKey", b"setCanSign:", {"arguments": {2: {"type": "Z"}}})
    r(b"TKTokenKeychainKey", b"setSuitableForLogin:", {"arguments": {2: {"type": "Z"}}})
    r(
        b"TKTokenWatcher",
        b"addRemovalHandler:forTokenID:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    }
                }
            }
        },
    )
    r(
        b"TKTokenWatcher",
        b"initWithInsertionHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    }
                }
            }
        },
    )
    r(
        b"TKTokenWatcher",
        b"setInsertionHandler:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    }
                }
            }
        },
    )
finally:
    objc._updatingMetadata(False)
expressions = {}

# END OF FILE



import objc

from PyObjCTools.TestSupport import TestCase, min_os_level
import CryptoTokenKit


class TestTkSmartCardATR(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(CryptoTokenKit.TKSmartCardProtocol)

    @min_os_level("10.10")
    def testClasses(self):
        self.assertIsInstance(
            CryptoTokenKit.TKSmartCardATRInterfaceGroup, objc.objc_class
        )
        self.assertIsInstance(CryptoTokenKit.TKSmartCardATR, objc.objc_class)

    @min_os_level("10.10")
    def testMethods(self):
        self.assertArgIsBlock(CryptoTokenKit.TKSmartCardATR.initWithSource_, 0, b"i@")

    @min_os_level("10.10")
    def testConstants(self):
        self.assertEqual(CryptoTokenKit.TKSmartCardProtocolNone, 0)
        self.assertEqual(CryptoTokenKit.TKSmartCardProtocolT0, 1)
        self.assertEqual(CryptoTokenKit.TKSmartCardProtocolT1, 2)
        self.assertEqual(CryptoTokenKit.TKSmartCardProtocolT15, 1 << 15)
        self.assertEqual(CryptoTokenKit.TKSmartCardProtocolAny, (1 << 16) - 1)
